@@ -2,7 +2,6 @@
 #'
 #' This function allows to perform the pivotal relabelling procedure described in Egidi et al. (2018) and to obtain the relabelled posterior estimates.
 #' @param mcmc The output of the MCMC sampling from \code{piv_MCMC}.
-#' @param nMC The number of total MCMC iterations (given in input to the \code{piv_MCMC} function, or any function suited for MCMC sampling).
 #'
 #'@details
 #'Prototypical models in which the label switching problem arises
@@ -76,20 +75,15 @@
 #' @return This function gives the relabelled posterior estimates--both mean and medians--obtained from the Markov chains of the MCMC sampling.
 #'
 #' \item{\code{final_it}}{The final number of valid MCMC iterations,
-#' as explained in Details}
-#' \item{\code{final_it_p}}{\code{final_it/nMC}}
-#' \item{\code{mu_rel_mean}}{ \code{k}-vector (in case of univariate mixture)
-#' or \code{k x 2}
-#' matrix (in case of bivariate mixture) of estimated posterior means for the mean parameters.}
-#' \item{\code{mu_rel_median}}{ \code{k}-vector (in case of univariate mixture)
-#' or \code{k x 2}
-#' matrix (in case of bivariate mixture) of estimated posterior medians for the mean parameters.}
-#' \item{\code{mu_rel}}{\code{final_it x k} matrix (in case of univariate mixtures)
-#' or \code{final_it x 2 x k} array (in case of bivariate mixtures)
-#' for the complete relabelled chains}
-
+#' as explained in Details.}
+#' \item{\code{final_it_p}}{The proportion of final valid MCMC iterations.}
+#' \item{\code{rel_mean}}{The relabelled chains of the means: a \code{final_it} \eqn{\times k} matrix for univariate data,
+#' or a \code{final_it} \eqn{\times 2 \times k} array for bivariate data.}
+#' \item{\code{rel_sd}}{The relabelled chains of the sd's: a \code{final_it} \eqn{\times k} matrix for univariate data,
+#' or a \code{final_it} \eqn{\times 2} matrix for bivariate data.}
+#' \item{\code{rel_weight}}{The relabelled chains of the weights: a \code{final_it} \eqn{\times k} matrix.}
 #'
-#' @author Leonardo Egidi \url{legidi@units.it}
+#' @author Leonardo Egidi \email{legidi@units.it}
 #' @references Egidi, L., Pappadà, R., Pauli, F. and Torelli, N. (2018). Relabelling in Bayesian Mixture
 #'Models by Pivotal Units. Statistics and Computing, 28(4), 957-969.
 #' @examples
@@ -101,12 +95,13 @@
 #' k   <- 3
 #' p   <- rep(1/k,k)
 #' x   <- 3
-#' stdev <- cbind(rep(1,k), rep(200,k))
+#' stdev <- cbind(rep(1,k), rep(20,k))
 #' Mu    <- seq(-trunc(k/2)*x,trunc(k/2)*x,length=k)
 #' W     <- c(0.2,0.8)
-#' sim   <- piv_sim(N,k,Mu,stdev,W=W)
+#' sim   <- piv_sim(N = N, k = k, Mu = Mu,
+#'                  stdev = stdev, W=W)
 #' res   <- piv_MCMC(y = sim$y, k =k, nMC = nMC)
-#' rel   <- piv_rel(mcmc=res, nMC = nMC)
+#' rel   <- piv_rel(mcmc=res)
 #'}
 #'
 #' #Bivariate simulation
@@ -118,15 +113,17 @@
 #' M2  <- c(25.5,.1)
 #' M3  <- c(49.5,8)
 #' Mu  <- matrix(rbind(M1,M2,M3),c(k,2))
-#' stdev <- cbind(rep(1,k), rep(200,k))
-#' Sigma.p1 <- matrix(c(stdev[1,1],0,0,stdev[1,1]),
+#' sds <- cbind(rep(1,k), rep(20,k))
+#' Sigma.p1 <- matrix(c(sds[1,1]^2,0,0,sds[1,1]^2),
 #'                    nrow=2, ncol=2)
-#' Sigma.p2 <- matrix(c(stdev[1,2],0,0,stdev[1,2]),
+#' Sigma.p2 <- matrix(c(sds[1,2]^2,0,0,sds[1,2]^2),
 #'                    nrow=2, ncol=2)
 #' W <- c(0.2,0.8)
-#' sim <- piv_sim(N,k,Mu,stdev,Sigma.p1,Sigma.p2,W)
+#' sim <- piv_sim(N = N, k = k, Mu = Mu,
+#'                Sigma.p1 = Sigma.p1,
+#'                Sigma.p2 = Sigma.p2, W = W)
 #' res <- piv_MCMC(y = sim$y, k = k, nMC = nMC)
-#' rel <- piv_rel(mcmc = res, nMC = nMC)
+#' rel <- piv_rel(mcmc = res)
 #' piv_plot(y=sim$y, mcmc=res, rel_est = rel, type="chains")
 #' piv_plot(y=sim$y, mcmc=res, rel_est = rel,
 #'          type="hist")
@@ -134,10 +131,24 @@
 #'
 #' @export
 
-piv_rel<-function(mcmc, nMC ){
-  N <- dim(mcmc$z)[1]
-  k <- dim(mcmc$z)[2]
-  mu_switch <- mcmc$mu_switch
+piv_rel<-function(mcmc){
+
+  ### checks
+
+
+  ###
+
+
+  N <- dim(mcmc$groupPost)[2]
+  if (length(dim(mcmc$mcmc_mean_raw))==3){
+  k <- dim(mcmc$mcmc_mean)[3]
+  }else{
+  k <- dim(mcmc$mcmc_mean_raw)[2]
+  }
+  nMC <- dim(mcmc$mcmc_mean_raw)[1]
+  mu_switch <- mcmc$mcmc_mean
+  tau_switch <- mcmc$mcmc_sd
+  prob.st_switch <- mcmc$mcmc_weight
   group <-  mcmc$groupPost
   pivots <- mcmc$pivots
   Mu <- mcmc$Mu
@@ -168,11 +179,23 @@ piv_rel<-function(mcmc, nMC ){
     mu_rel_mean       <- c()
     mu_rel_median_tr  <- c()
     mu_rel_mean_tr    <- c()
+    tau_rel_median     <- c()  #vector of length k
+    tau_rel_mean       <- c()
+    tau_rel_median_tr  <- c()
+    tau_rel_mean_tr    <- c()
+    weights_rel_median     <- c()  #vector of length k
+    weights_rel_mean       <- c()
+    weights_rel_median_tr  <- c()
+    weights_rel_mean_tr    <- c()
     groupD2           <- groupD[contD==0,]
     mu_switchD        <- mu_switch[contD==0,]
+    tau_switchD       <- tau_switch[contD==0,]
+    prob.st_switchD   <- prob.st_switch[contD==0,]
     true.iterD2       <- sum(contD==0)
     Final_It          <- true.iterD2/nMC
     mu_rel_complete   <- matrix(NA,true.iterD2, k)
+    tau_rel_complete  <- matrix(NA, true.iterD2, k)
+    weights_rel_complete <- matrix(NA, true.iterD2, k)
 
 
     if (true.iterD2!=0){
@@ -183,39 +206,61 @@ piv_rel<-function(mcmc, nMC ){
             mu_rel_complete[m,j] <-
               mu_switchD[m,
                 vect_rel[groupD2[m, pivots[j]]] ]
+            tau_rel_complete[m,j] <-
+              tau_switchD[m,
+                         vect_rel[groupD2[m, pivots[j]]] ]
+            weights_rel_complete[m,j] <-
+              prob.st_switchD[m,
+                         vect_rel[groupD2[m, pivots[j]]] ]
+
           }
         }
       }else{
-        mu_rel_median <- rep(NA,k)
-        mu_rel_mean   <- rep(NA,k)
+        stop("The number of MCMC iterations is too low, try increasing the argument nMC when you use the piv_MCMC function.")
+        #mu_rel_median <- rep(NA,k)
+        #mu_rel_mean   <- rep(NA,k)
       }
 
       mu_rel_median  <- apply(mu_rel_complete, 2, median)
       mu_rel_mean    <- apply(mu_rel_complete, 2, mean)
       mu_rel_median_tr  <- t(mu_rel_median)
       mu_rel_mean_tr    <- t(mu_rel_mean)
+      tau_rel_median  <- apply(tau_rel_complete, 2, median)
+      tau_rel_mean    <- apply(tau_rel_complete, 2, mean)
+      tau_rel_median_tr  <- t(tau_rel_median)
+      tau_rel_mean_tr    <- t(tau_rel_mean)
+      weights_rel_median  <- apply(weights_rel_complete, 2, median)
+      weights_rel_mean    <- apply(weights_rel_complete, 2, mean)
+      weights_rel_median_tr  <- t(weights_rel_median)
+      weights_rel_mean_tr    <- t(weights_rel_mean)
 
   }else{
     k <- dim(mu_switch)[3]
     mu_rel_median  <- array(NA,c(2,k))
     mu_rel_mean    <- array(NA,c(2,k))
+    weights_rel_median <- c()
+    weights_rel_mean <- c()
     groupD2        <- groupD[contD==0,]
     mu_switchD     <- mu_switch[contD==0,,]
+    prob.st_switchD     <- prob.st_switch[contD==0,]
     true.iterD2    <- sum(contD==0)
     Final_It       <- true.iterD2/nMC
     mu_rel_complete  <- array(NA, dim=c(true.iterD2, 2,k))
-
+    weights_rel_complete <- array(NA, dim=c(true.iterD2,k))
 
       if (true.iterD2!=0){
         for (m in 1:true.iterD2){
           for (j in 1:k){
             mu_rel_complete[m,,j] <-
               mu_switchD[m,,groupD2[m, pivots[j]]]
+            weights_rel_complete[m,j] <-
+              prob.st_switchD[m,groupD2[m, pivots[j]]]
           }
         }
       }else{
-        mu_rel_median <- matrix(NA,c(2,k))
-        mu_rel_mean   <- matrix(NA,c(2,k))
+        stop("The number of MCMC iterations is too low, try increasing the argument nMC when you use the piv_MCMC function.")
+        #mu_rel_median <- matrix(NA,c(2,k))
+        #mu_rel_mean   <- matrix(NA,c(2,k))
 
       }
 
@@ -237,17 +282,25 @@ piv_rel<-function(mcmc, nMC ){
  for (h in 1:nrow(mu_rel_complete)){
   mu_rel_complete[h,1,] <- mu_rel_complete[h,1, ind[h,1,]]
   mu_rel_complete[h,2,] <- mu_rel_complete[h,2, ind[h,2,]]
-      }
+  #weights_rel_complete[h,1] <- weights_rel_complete[h,ind[h,1,]]
+  #weights_rel_complete[h,2] <- weights_rel_complete[h,ind[h,2,]]
+  }
   mu_rel_median    <- apply(mu_rel_complete, c(2,3), median)
   mu_rel_mean      <- apply(mu_rel_complete, c(2,3), mean)
   mu_rel_median_tr <- t(mu_rel_median)
   mu_rel_mean_tr   <- t(mu_rel_mean)
+  weights_rel_median    <- apply(weights_rel_complete, 2, median)
+  weights_rel_mean      <- apply(weights_rel_complete, 2, mean)
+  weights_rel_median_tr <- t(weights_rel_median)
+  weights_rel_mean_tr   <- t(weights_rel_mean)
+  tau_rel_median        <- apply(tau_switch, 2, median)
+  tau_rel_complete      <- tau_switch
     }
 
  return(list( final_it = true.iterD2,
-              mu_rel_median = mu_rel_median_tr,
-              mu_rel_mean = mu_rel_mean_tr,
-              mu_rel = mu_rel_complete,
+              rel_mean = mu_rel_complete,
+              rel_sd = tau_rel_complete,
+              rel_weight = weights_rel_complete,
               final_it_p = Final_It))
 }
 
